unit uMHA_DLL_GAF_List;

interface

uses
  TRPCb,
  Classes,
  dialogs,
  Controls,
  SysUtils;

type
  TGAF_List = class
  private
    FGAFList: TStringList;
    FRatingIsDue: BOOLEAN;
    FPreviousRatingsExist: BOOLEAN;
    FNumberOfRatings: Integer;
    FRatingsCanBeGraphed: Boolean;
    procedure Clear;
    procedure SetRatingsCanBeGraphed;
  protected
  public
    destructor Destroy; override;
    constructor Create;
    procedure LoadRatingList(Broker: TRPCBroker; ADFN: string);
    procedure LoadByIndex(AIndex: integer);
    procedure LoadByIENS(AIENS: string);
    //property RatingIsDue: BOOLEAN read FRatingIsDue write FRatingIsDue;
    property PreviousRatingsExist: BOOLEAN read FPreviousRatingsExist write
      FPreviousRatingsExist;
    property NumberOfRatings: Integer read FNumberOfRatings write
      FNumberOfRatings;
    property RatingsCanBeGraphed: BOOLEAN read FRatingsCanBeGraphed write
      FRatingsCanBeGraphed;
  end; { TGAF_List }

implementation

uses
  uIA_Const,
  uMHA_DLL_Const,
  uIA_Patient,
  uMHA_DLL_GAF_IO,
  uMHA_DLL_GAF,
  uIA_Utils;

destructor TGAF_List.Destroy;
begin
  if FGAFList <> nil then
    FGAFList.Free;
  if GAF <> nil then
    GAF.Free;
  inherited Destroy;
end;

constructor TGAF_List.Create;
begin
  inherited Create;
  GAF := TGAF.Create;
  FGAFList := TStringList.Create;
end;

procedure TGAF_List.LoadRatingList(Broker: TRPCBroker; ADFN: string);
var
  TempID                      : string;
begin
  TempID := ADFN;
  Clear;

  PiecesToList(ReadPreviousGAFRatings(Broker, TempID), RECORD_END, FGAFList);

  if (FGAFList.Count = 0)
    or (FGAFList[0] = BAD_DATA) then
    FGAFList.Clear //NB: Other properties were cleared at beginning of procedure
  else
    begin
      FNumberOfRatings := FGAFList.Count;
      FPreviousRatingsExist := FGAFList.Count > 0;
      SetRatingsCanBeGraphed;
    end;

end;

procedure TGAF_List.LoadByIndex(AIndex: integer);
var
  tmpIndex                    : integer;
begin
  tmpIndex := AIndex;

  if (tmpIndex < 0) or (tmpIndex > FGAFList.Count - 1) then
    Exit;

  GAF.Load(FGAFList[tmpIndex]);

end;

procedure TGAF_List.LoadByIENS(AIENS: string);
var
  tmpIndex                    : integer;
  tmpIENS                     : string;
begin
  tmpIENS := AIENS;
  tmpIndex := FGAFList.IndexOfName(tmpIENS);
  if (tmpIndex > -1) then
    LoadByIndex(tmpIndex);
end;


procedure TGAF_List.Clear;
begin
  FGAFList.Clear;
  FRatingIsDue := True;
  FPreviousRatingsExist := False;
  FNumberOfRatings := 0;
  FRatingsCanBeGraphed := False;
end;

procedure TGAF_List.SetRatingsCanBeGraphed;
var
  NumNoErrors, i              : integer;
begin
  //To be able to graph ratings, there must be at least 2 ratings not
  //marked in error

  //Since FRatingsCanBeGraphed is set to False in the Clear procedure...
  if not (FNumberOfRatings > 1) then
    Exit;

  NumNoErrors := 0;

  for i := 0 to FNumberOfRatings - 1 do
    if not (Piece(FGAFList[i], U, 5) = 'ERROR') then
      NumNoErrors := NumNoErrors + 1;

  FRatingsCanBeGraphed := (NumNoErrors > 1);

end;

end.

