unit uMHA_DLL_Instrument_OPS;

interface

uses
  Classes,
  Trpcb,
  SysUtils,
  dialogs,
  uIA_Instrument;

type
  TDLL_Instrument_OPS = class
  private
    function LoadQuestionAndIntroLists(Broker: TRPCBroker; var aInstrument: TInstrument): Boolean;
    function LoadQuestionChoices(Broker: TRPCBroker; var aInstrument: TInstrument): Boolean;
    procedure LoadSectionList(Broker: TRPCBroker; var aInstrument: TInstrument);
    procedure LoadScales(Broker: TRPCBroker; var aInstrument: TInstrument);
    procedure LoadRuleList(Broker: TRPCBroker; var aInstrument: TInstrument);
    procedure LoadKeys(Broker: TRPCBroker; var aInstrument: TInstrument);
    procedure InitializeListOfAnswers(var aInstrument: TInstrument);
  protected
  public
    function Load(Broker: TRPCBroker; var aInstrument: TInstrument): Boolean;
    function LoadDescriptiveProperties(Broker: TRPCBroker; var aInstrument: TInstrument): Boolean;

  end; // TInstrument_OPS

var
  DLL_Instrument_OPS          : TDLL_Instrument_OPS;

implementation

uses
  System.Contnrs,
  uIA_Const,
  uIA_Utils,
  uIA_Question,
  uIA_Answer,
  uIA_Question_IO,
  uIA_Instrument_IO,
  uIA_SectionLabel_IO,
  uIA_MultiChoice_IO,
  uIA_Rule_IO,
  uIA_Scales_IO,
  uIA_Keys_IO;


function TDLL_Instrument_OPS.LoadDescriptiveProperties(Broker: TRPCBroker;
  var aInstrument: TInstrument): BOOLEAN;
begin
  Result := False;

  if InstrumentReadDescriptiveProperties(Broker, aInstrument) = BAD_DATA then
    begin
      ShowMessage('Unable to load descriptive properties of ' + aInstrument.InstrumentName + '.');
      Exit;
    end
  else
    Result := True;

end; //LoadDescriptiveProperties


function TDLL_Instrument_OPS.Load(Broker: TRPCBroker; var aInstrument: TInstrument): BOOLEAN;
begin
  Result := True; //the default

  if not LoadDescriptiveProperties(Broker, aInstrument) or
    not LoadQuestionAndIntroLists(Broker, aInstrument) or
    not LoadQuestionChoices(Broker, aInstrument) then
    begin
      Result := False;
      Exit;
    end;

  LoadRuleList(Broker, aInstrument);
  LoadSectionList(Broker, aInstrument);
  LoadScales(Broker, aInstrument);
  LoadKeys(Broker, aInstrument);
  InitializeListOfAnswers(aInstrument);
end;


procedure TDLL_Instrument_OPS.InitializeListOfAnswers(var aInstrument: TInstrument);
var
  i                           : Integer;
  AAnswer                     : TAnswer;
begin
  aInstrument.ListOfAnswers.Clear;

  for i := 0 to aInstrument.ListOfQuestions.Count - 1 do
    begin
      AAnswer := TAnswer.Create;
      AAnswer.QuestionID := TQuestion(aInstrument.ListOfQuestions[i]).QuestionID;
      AAnswer.AnswerValue := NOT_ASKED;
      AAnswer.AnswerText := NOT_ASKED;
      AAnswer.IsMultiChoice := False;
      AAnswer.Designator := TQuestion(aInstrument.ListOfQuestions[i]).Designator;
      AAnswer.AnswerLegacyValue := '';
      aInstrument.ListOfAnswers.Add(AAnswer);
    end; // for i
end;

function TDLL_Instrument_OPS.LoadQuestionAndIntroLists(Broker: TRPCBroker;
  var aInstrument: TInstrument): Boolean;
begin
  Result := False;

  aInstrument.ListOfQuestions.Clear;
  aInstrument.ListOfIntroductions.Clear;

  // See uIA_Question_IO
  if ReadQuestionAndIntroLists(Broker, aInstrument) = OK then
    Result := True
  else
    ShowMessage('Unable to load questions and introductions for ' + aInstrument.InstrumentName + '.');
end;


function TDLL_Instrument_OPS.LoadQuestionChoices(Broker: TRPCBroker;
  var aInstrument: TInstrument): Boolean;
// This adds the multiple choice items to each question in the ListOfQuestions,
// although some questions will not need this service.
var
  i, j                        : Integer;
  TempStr                     : string;
  TempList                    : TStringList;

begin
  Result := True;
  TempList := TStringList.Create;

  PiecesToList(ReadChoiceList(aInstrument.InstrumentName, Broker), RECORD_END, TempList);

// Raw data from Vista looks like:
// 469^1^100098^One^^1
// 469^2^100099^Two^^2
// 469^3^100100^Three^^3
// 469^4^100101^Four^^4
// 480^1^100165^Low^
// 480^2^100166^Medium^
// 480^3^100752^ high^
// 766^1^237^No^N
// 766^2^241^Yes^Y
// 766^3^1251^Declined to answer^
// 766^4^1252^Failed to ascertain^
// 777^1^1260^Very irregular answer^
// 777^2^1261^Second irregular answer^
// 777^3^1262^Unique response^
// 777^4^1263^Very unusual response^

  if (TempList[0] = BAD_DATA) then
    begin
      ShowMessage('Unable to load question choices for ' + aInstrument.InstrumentName + '.');
      Result := False;
      TempList.Free;
      Exit;
    end;

  for i := 0 to aInstrument.ListOfQuestions.Count - 1 do
    begin
      for j := 0 to TempList.count - 1 do
        begin
          if TQuestion(aInstrument.ListOfQuestions[i]).ChoiceTypeID =
            Piece(TempList[j], U, 1) then
            begin

              TempStr := Piece(TempList[j], U, 2) + U; // Designator/Sequence
              TempStr := TempStr + Piece(TempList[j], U, 3) + U; // ChoiceID
              TempStr := TempStr + Piece(TempList[j], U, 4) + U; // Text
              TempStr := TempStr + Piece(TempList[j], U, 5); // Legacy Value

              TQuestion(aInstrument.ListOfQuestions[i]).MultiChoice.Choices.Add(TempStr);
              TQuestion(aInstrument.ListOfQuestions[i]).MultiChoice.NumberOfPicks :=
                TQuestion(aInstrument.ListOfQuestions[i]).qMax;
              TQuestion(aInstrument.ListOfQuestions[i]).MultiChoice.NumberOfChoices :=
                TQuestion(aInstrument.ListOfQuestions[i]).MultiChoice.NumberOfChoices + 1;
            end;

        end; // for j

    end; // for i

  TempList.Free;

end;

procedure TDLL_Instrument_OPS.LoadRuleList(Broker: TRPCBroker;
  var aInstrument: TInstrument);
// This adds the business rule to the ListOfRules.
begin
  aInstrument.ListOfRules.Clear;

  if not ReadRules(Broker, aInstrument) then // uIA_Rule_IO
    ShowMessage('Unable to load rules for ' + aInstrument.InstrumentName + '.');

end;

procedure TDLL_Instrument_OPS.LoadSectionList(Broker: TRPCBroker;
  var aInstrument: TInstrument);
begin
  aInstrument.ListOfSections.Clear;

  // See uIA_SectionLabel_IO
  if not ReadSectionList(Broker, False {NeedIEN}, aInstrument) then
    ShowMessage('Unable to load section labels for ' + aInstrument.InstrumentName + '.');
end;

procedure TDLL_Instrument_OPS.LoadScales(Broker: TRPCBroker;
  var aInstrument: TInstrument);
begin
  aInstrument.ListOfScales.Clear;

  // See uIA_Scales_IO
  if not ReadScales(Broker, aInstrument) then //
    ShowMessage('Unable to load scales for ' + aInstrument.InstrumentName + '.');
end;

procedure TDLL_Instrument_OPS.LoadKeys(Broker: TRPCBroker;
  var aInstrument: TInstrument);
begin
  aInstrument.ListOfKeys.Clear;

  if not ReadScoringKeys(Broker, aInstrument) then // uIA_Keys_IO
    ShowMessage('Unable to load the scoring keys for ' + aInstrument.InstrumentName);
end;

end.

