unit uMHA_DLL_Vista_IO;

interface

uses
  Classes,
  Windows,
  dialogs,
  SysUtils,
  Trpcb,
  fIA_Dialogs,
  uIA_Utils,
  uIA_Instrument,
  uIA_Const,
  uMHA_DLL_Const;

function CorrectVersion(Broker: TRPCBroker): Boolean;

function SaveAdministration(Broker: TRPCBroker;
  WhichInstrument, PatientDFN,
  OrderedByDUZ, AdministeredByDUZ, LocationIEN,
  AdminDate, // FileMan format
  NumberOfQuestions: string): string;
procedure SaveScore(anAdministrationID: string; Broker: TRPCBroker);
function SendHL7Message(Broker: TRPCBroker;
  anAdministrationID: string): string;

function WriteAnswersToTemp(Broker: TRPCBroker; PatientDFN: string;
  aInstrument: TInstrument): Boolean;
function ReadAnswersFromTempForObject(Broker: TRPCBroker; PatientDFN: string;
  var aInstrument: TInstrument): Boolean;
function ReadAnswersFromTempForList(Broker: TRPCBroker;
  PatientDFN, InstrumentName: string; var AnswerList: TStringList): Boolean;
function RemoveTemp(Broker: TRPCBroker;
  InstrumentName, PatientDFN: string): Boolean;

function WriteAnswers(Broker: TRPCBroker; aAdminID: string; AnswerList: TStringList): Boolean;

function ReadStaffProperties(Broker: TRPCBroker; ADUZ: string; AKey: string): string;
procedure ReturnContextToCPRS(Broker: TRPCBroker);

implementation

uses
  System.Contnrs,
  uIA_Answer;

const
 //Following constants borrowed from CPRS

  TX_VER1                     = 'This is version ';
  TX_VER3                     = #13 + 'The running server version is ';
  TX_VER_REQ                  = ' version server is required.';
  TX_VER_OLD                  = #13 + 'It is strongly recommended that you upgrade.';
  TX_VER_OLD2                 = #13 +
    'The program cannot be run online until the CLIENT is upgraded.';
  TX_VER_NEW                  = #13 +
    'The program cannot be run online until the SERVER is upgraded.';

 { values that can be passed to FileVersionValue }
  FILE_VER_FILEVERSION        = '\StringFileInfo\040904E4\FileVersion';
  FILE_VER_INTERNALNAME       = '\StringFileInfo\040904E4\InternalName';
  FILE_VER_PRODUCTVERSION     = '\StringFileInfo\040904E4\ProductVersion';

procedure ReturnContextToCPRS(Broker: TRPCBroker);
begin
  with Broker do
    begin
      ClearParameters := True;
      ClearResults := True;
      CreateContext('OR CPRS GUI CHART');
    end;
end;


function SaveAdministration(Broker: TRPCBroker;
  WhichInstrument, PatientDFN,
  OrderedByDUZ, AdministeredByDUZ, LocationIEN,
  AdminDate,
  NumberOfQuestions: string): string;
begin

  with Broker do
    begin
      ClearParameters := True;
      ClearResults := True;
      CreateContext('YS BROKER1');
      RemoteProcedure := 'YTQ SAVE';
      Param[0].Mult['"FILEN"'] := '601.84';
      Param[0].Mult['"1"'] := '.01^NEW^1';
      Param[0].Mult['"2"'] := '1^`' + PatientDFN;
      Param[0].Mult['"3"'] := '2^' + WhichInstrument;
      Param[0].Mult['"4"'] := '3^' + AdminDate; // FileMan format
      Param[0].Mult['"5"'] := '4^NOW'; // DateSaved
      Param[0].Mult['"6"'] := '5^`' + OrderedByDUZ;
      Param[0].Mult['"7"'] := '6^`' + AdministeredByDUZ;
      Param[0].Mult['"8"'] := '7^N'; //IsSigned
      Param[0].Mult['"9"'] := '8^Y'; //IsComplete
      Param[0].Mult['"10"'] := '9^' + NumberOfQuestions;
      if (LocationIEN = '') or (LocationIEN = '0') or
        (LocationIEN = ' ') then // null causes crash
//**
//ShowMessage('Location is blank and number of questions is : '+ NumberOfQuestions)
      else
        begin
//      ShowMessage('Location is '+ LocationIEN + #13 +
//       'Number of questions is '+ NumberOfQuestions);
          Param[0].Mult['"11"'] := '13^`' + LocationIEN; //Location
        end;
      Param[0].PType := List;
      Call;

      if Results[0] = DATA then
        Result := Piece(Results[1], U, 2) // IEN of Administration
      else
        Result := BAD_DATA + ' ' + Results[1];

      // Needed for DLL, in case there is a crash
      CreateContext('OR CPRS GUI CHART');
    end; // with Broker
end;

procedure SaveScore(anAdministrationID: string; Broker: TRPCBroker);
// All the work is done on the M-side.
begin
  with Broker do
    begin
      ClearParameters := True;
      ClearResults := True;
      CreateContext('YS BROKER1');
      RemoteProcedure := 'YTQ SCORE SAVE';
      Param[0].Mult['"AD"'] := anAdministrationID;
      Param[0].PType := List;

      Call;

      CreateContext('OR CPRS GUI CHART');
    end;
end;


function SendHL7Message(Broker: TRPCBroker;
  anAdministrationID: string): string;
begin
  result := DATA;
(* // No longer used.


  //NAME: YTQ HL7
  //    Input: AD as administration ID
  //    Output : DATA vs ERROR
  result := '';
  with Broker do
    begin
      ClearParameters := True;
      ClearResults := True;
      CreateContext('YS BROKER1');
      RemoteProcedure := 'YTQ HL7';
      Param[0].Mult['"AD"'] := anAdministrationID;
      Param[0].PType := List;
      Call;
      Result := Results[0];

      // Needed for DLL, in case there is a crash
      CreateContext('OR CPRS GUI CHART');
    end;
*)
end;


function WriteAnswersToTemp(Broker: TRPCBroker; PatientDFN: string;
  aInstrument: TInstrument): BOOLEAN;
// M-side saves User.DUZ with data.
// For Answers with text longer than 200 characters, send it to Vista in
// chuncks of 200; YS = 1.1 for first chunk, YS = 1.2 for second chunk,
// YS = 1.3 for third, etc.

var
  i, j,
    CountStr                  : integer;

  TempApp,
    TempStr,
    YS                        : string;
//LogFileStr: TStringList;

begin
  TempApp := Piece(GetAppHandle(Broker), '-', 1);
//LogFileStr:= TStringList.create;
  with Broker do
    begin
      ClearParameters := True;
      ClearResults := True;
      CreateContext('YS BROKER1');
      Remoteprocedure := 'YTQ SET SCRATCH';
      Param[0].Mult['"DFN"'] := PatientDFN;
      Param[0].Mult['"CODE"'] := aInstrument.InstrumentName;
      Param[0].Mult['"HANDLE"'] := TempApp;

      for i := 0 to aInstrument.ListOfAnswers.Count - 1 do
        begin
          with TAnswer(aInstrument.ListOfAnswers[i]) do
            begin
             (*
              AnswerText := 'This is test text that will be in excess of 200 characters ' +
                'to provide a sample of long text for the new API. This simple sentence will repeat. ' +
                '2 This is test text that will be in excess of 200 characters ' +
                'to provide a sample of long text for the new API. This simple sentence will repeat. ' +
                '3 This is test text that will be in excess of 200 characters ' +
                'to provide a sample of long text for the new API. This simple sentence will repeat. ' +
                '4 This is test text that will be in excess of 200 characters ' +
                'to provide a sample of long text for the new API. This simple sentence will repeat. ' +
                '5 This is test text that will be in excess of 200 characters ' +
                'to provide a sample of long text for the new API. This simple sentence will repeat. ' +
                '6 This is test text that will be in excess of 200 characters ' +
                'to provide a sample of long text for the new API. This simple sentence will repeat. ';
               *)
              TempStr := '';
              TempStr := RemoveControlChar(AnswerText);

              TempStr := StringToPieces(TempStr, U, 200);
              CountStr := NumberOfPieces(TempStr, U);
//LogFileStr.add(QuestionID + U +
//                    AnswerValue + U +
//                    AnswerLegacyValue + U +
//                    BoolToMStr(IsMultiChoice) + U);
              if CountStr = 0 then // AnswerText is less than 200 characters
                begin
                  YS := '"' + IntToStr(i + 1) + '"';

                  Param[0].Mult[YS] := QuestionID + U +
                    AnswerValue + U +
                    AnswerLegacyValue + U +
                    BoolToMStr(IsMultiChoice) + U;

                  YS := '"' + IntToStr(i + 1) + '.1"';
                  Param[0].Mult[YS] := TempStr;
                end
              else
              // Case where AnswerText is longer than 200 characters
                begin
                  YS := '"' + IntToStr(i + 1) + '"';
                  Param[0].Mult[YS] := QuestionID + U +
                    AnswerValue + U +
                    AnswerLegacyValue + U +
                    BoolToMStr(IsMultiChoice) + U;

                  for j := 1 to CountStr do
                    begin
                      YS := '"' + IntToStr(i + 1) + '.' + IntToStr(j) + '"';

                      Param[0].Mult[YS] := Piece(TempStr, U, j);
                    end;
                end; // else
            end; // with TAnswer
        end; // for i
//LogFileStr.SaveToFile('ScratchFileSet.txt');
//LogFileStr.Free;
      Param[0].PType := List;
      Call;

      Result := True;

      // Needed for DLL, in case there is a crash
      CreateContext('OR CPRS GUI CHART');
    end; // with broker
end;

function RemoveTemp(Broker: TRPCBroker;
  InstrumentName, PatientDFN: string): BOOLEAN;
var
  TempApp                     : string;
begin

  if InstrumentName = 'GAF' then
    begin
      result := True;
      Exit;
    end;

  TempApp := Piece(GetAppHandle(Broker), '-', 1);

  with Broker do
    begin
      ClearParameters := True;
      ClearResults := True;
      CreateContext('YS BROKER1');
      RemoteProcedure := 'YTQ KILL SCRATCH';
      Param[0].Mult['"DFN"'] := PatientDFN;
      if InstrumentName <> '' then
        Param[0].Mult['"CODE"'] := InstrumentName;
      Param[0].Mult['"HANDLE"'] := TempApp;
      Param[0].PType := List;
      Call;
    end;

  result := true;
end;


function ReadAnswersFromTempForList(Broker: TRPCBroker;
  PatientDFN, InstrumentName: string; var AnswerList: TStringList): Boolean;
// This reads the scratch file; results are used to save the administration.
var
  i, j                        : Integer;
  TempStr, TempApp,
    OldQID, NewQID            : string;
//LogFileStr: TStringList;
begin
//LogFileStr:= TStringList.create;
//  Result := False;
  TempApp := Piece(GetAppHandle(Broker), '-', 1);
  AnswerList.Clear;

  with Broker do
    begin
      ClearParameters := True;
      ClearResults := True;
      CreateContext('YS BROKER1');
      RemoteProcedure := 'YTQ GET SCRATCH';
      Param[0].Mult['"DFN"'] := PatientDFN;
      Param[0].Mult['"CODE"'] := InstrumentName;
      Param[0].Mult['"HANDLE"'] := TempApp;
      Param[0].PType := List;
      Call;

      if Pos('ok-none', Results[1]) > 0 then
        begin
          Result := True;
          Exit;
        end;

     // Output: QuestionID^AnswerValue^LegacyValue^IsMultiChoice
     //         QuestionID^AnswerText (first 200 char)
     //         QuestionID^AnswerText (next 200 char)

      OldQID := '';
      j := 0;

      for i := 1 to Results.Count - 1 do
        begin
//LogFileStr.add(Results[i]);
          NewQID := Piece(Results[i], U, 1);
          // For long text answers the next chunk of 200 chars are in
          // the next results[i].

          if (NewQID = OldQID) then
            begin
              TempStr := AnswerList[j - 1];
              SetPiece(TempStr, U, 3, Piece(TempStr, U, 3) + Piece(Results[i], U, 2));
              AnswerList[j - 1] := TempStr;
              TempStr := '';
            end
          else
// AnswerList is in format:
//   QuestionID + U +
//   AnswerValue(ChoiceID) + U + AnswerText + U + LegacyValue + BoolToMStr(IsMultiChoice) + U
            begin
              // Question ID
              AnswerList.Add(NewQID + U +
              // AnswerValue: This is the ChoiceID for a Multi-Choice question
                Piece(Results[i], U, 2) + U +
              // Place holder for AnswerText
                U +
              // Legacy Value
                Piece(Results[i], U, 3) + U +
              // IsMultiChoice
                Piece(Results[i], U, 4) + U);
//ShowMessage(Results[i]);
              Inc(j);
              OldQID := NewQID;
            end;
        end; //for i

      Result := True;
      // Needed for DLL, in case there is a crash
      CreateContext('OR CPRS GUI CHART');
    end; // with broker
//LogFileStr.SaveToFile('ScratchFileGet.txt');
//LogFileStr.Free;
end;


function ReadAnswersFromTempForObject(Broker: TRPCBroker; PatientDFN: string;
  var aInstrument: TInstrument): BOOLEAN;
var
  i, k                        : Integer;
  TempApp,
    OldQID, NewQID            : string;
begin

//  Result := False;
  gvPriorAnswers := False;

  TempApp := Piece(GetAppHandle(Broker), '-', 1);

  with Broker do
    begin
      ClearParameters := True;
      ClearResults := True;
      CreateContext('YS BROKER1');

      RemoteProcedure := 'YTQ GET SCRATCH';
      Param[0].Mult['"DFN"'] := PatientDFN;
      Param[0].Mult['"CODE"'] := aInstrument.InstrumentName;
      Param[0].Mult['"HANDLE"'] := TempApp;
      Param[0].PType := List;
      Call;

      Result := True;
      if Pos('ok-none', Results[1]) > 0 then
        Exit;

      gvPriorAnswers := True;
     // Output: QuestionID^AnswerValue^LegacyValue^IsMultiChoice
     //         QuestionID^AnswerText (first 200 char)
     //         QuestionID^AnswerText (next 200 char)

      OldQID := '';

      for i := 1 to Results.Count - 1 do
        begin

          NewQID := Piece(Results[i], U, 1);
          // For long text answers the next chunk of 200 chars are in
          // the next results[i].

          if NewQID = OldQID then
            begin
              for k := 0 to aInstrument.ListofAnswers.Count - 1 do
                begin
                  if TAnswer(aInstrument.ListOfAnswers[k]).QuestionID = NewQID then
                    begin
                      TAnswer(aInstrument.ListOfAnswers[k]).AnswerText :=
                        TAnswer(aInstrument.ListOfAnswers[k]).AnswerText +
                        Piece(Results[i], U, 2);
                    end;
                end;
            end
          else
            begin
              for k := 0 to aInstrument.ListofAnswers.Count - 1 do
                begin
                  if TAnswer(aInstrument.ListOfAnswers[k]).QuestionID = NewQID then
                    begin
                      TAnswer(aInstrument.ListOfAnswers[k]).AnswerValue := Piece(Results[i], U, 2);
                      TAnswer(aInstrument.ListOfAnswers[k]).AnswerLegacyValue := Piece(Results[i], U, 3);
                      TAnswer(aInstrument.ListOfAnswers[k]).IsMultiChoice := StrToBool(Piece(Results[i], U, 4));
                      TAnswer(aInstrument.ListOfAnswers[k]).AnswerText := '';
                    end;
                end;

              OldQID := NewQID;
            end;
        end; //for i
(*
   for i := 0 to aInstrument.ListOfAnswers.Count -1 do
  ShowMessage(TAnswer(aInstrument.ListOfAnswers[i]).QuestionID+#13+
TAnswer(aInstrument.ListOfAnswers[i]).AnswerValue+#13+
TAnswer(aInstrument.ListOfAnswers[i]).AnswerText+#13+
TAnswer(aInstrument.ListOfAnswers[i]).AnswerLegacyValue);
*)
      // Needed for DLL, in case there is a crash
      CreateContext('OR CPRS GUI CHART');
    end; // with broker
end;


function WriteAnswers(Broker: TRPCBroker; aAdminID: string; AnswerList: TStringList): BOOLEAN;
// AnswerList is in format:
//   QuestionID + U +
//   AnswerValue(ChoiceID) + U + AnswerText + U + LegacyValue + U +
//   BoolToMStr(IsMultiChoice) + U
var
  i, j                        : integer;
  YS,
    TempStr                   : string;
//logStr:TStringList;
begin
//  Result := False;
//logStr := Tstringlist.create;

  with Broker do
    begin
      ClearParameters := True;
      ClearResults := True;
      CreateContext('YS BROKER1');
      Remoteprocedure := 'YTQ SET ANSWER ALL';

      YS := '"AD"';
      Param[0].Mult[YS] := aAdminID;

      for i := 0 to AnswerList.Count - 1 do
        begin
          if StrToBool(Piece(AnswerList[i], U, 5)) then // Multi-Choice
            begin
              YS := '"' + IntToStr(i + 1) + '"'; // Quotes are needed
              Param[0].Mult[YS] :=
                Piece(AnswerList[i], U, 1) + U + Piece(AnswerList[i], U, 2);
//logstr.add(YS + ' ' + Piece(AnswerList[i], U, 1) + U + Piece(AnswerList[i], U, 2));
            end
          else
          // Not multiChoice:
          // Date fields, edit, spin-edit, check-list (text of checked items)
            begin
              TempStr := '';
              // Remove any control characters in string

              TempStr := StringToPieces(RemoveControlChar(Piece(AnswerList[i], U, 3)), U, 200);

              YS := '"' + IntToStr(i + 1) + '"';
              Param[0].Mult[YS] :=
                Piece(AnswerList[i], U, 1);

              for j := 1 to NumberOfPieces(TempStr, U) do
                begin
                  // Eventhough this is not in the same format as above, do NOT
                  // change it.
                  YS := IntToStr(i + 1) + ',' + IntToStr(j); // No quotes. right
                  Param[0].Mult[YS] := Piece(TempStr, U, j);

                  //logstr.add(YS + ' ' + Piece(TempStr, U, j));
                end; // for j

            end; // MultiChoice
        end; // for i

      Param[0].PType := List;

      Call;

//logstr.SaveToFile('LogStr.txt');
//logstr.Free;
      Result := True;

      // Needed for DLL, in case there is a crash
      CreateContext('OR CPRS GUI CHART');
    end; // with broker
end;


function ReadStaffProperties(Broker: TRPCBroker;
  ADUZ: string; AKey: string): string;
// This call returns:
// 1) Name ^ DUZ
// 2) x^holds YSP Key  where x is blank or 1
// 3) location id ^ location name ^ location ID
var
  tmpDUZ, tmpKey              : string;
begin
  tmpDUZ := ADUZ;
  tmpKey := AKey;
  Result := BAD_DATA;

  if tmpDUZ = '' then
    Exit;

  with Broker do
    begin
      ClearParameters := True;
      ClearResults := True;
      CreateContext('YS BROKER1');
      RemoteProcedure := 'YTQ USERQ';

      Param[0].Mult['"DUZ"'] := tmpDUZ;
      if not (tmpKey = '') then
        Param[0].Mult['"KEY"'] := tmpKey;
      Param[0].PType := List;
      Call;

      if Results[0] <> DATA then
        Result := BAD_DATA
      else
        begin
          if Results.Count < 2 then
            Exit;
          Result := Piece(Results[2], U, 1);
        // Result should be either blank or 1. 1 means they have the key.
        end;

      // Needed for DLL, in case there is a crash
      CreateContext('OR CPRS GUI CHART');
    end; //with RPCBroker1

end;


function CompareVersion(const A, B: string): Integer;
var
  NumA, NumB                  : Integer;
begin
  NumA := (StrToInt(Piece(A, '.', 1)) * 16777216) +
    (StrToInt(Piece(A, '.', 2)) * 65536) +
    (StrToInt(Piece(A, '.', 3)) * 256) +
    StrToInt(Piece(A, '.', 4));
  NumB := (StrToInt(Piece(B, '.', 1)) * 16777216) +
    (StrToInt(Piece(B, '.', 2)) * 65536) +
    (StrToInt(Piece(B, '.', 3)) * 256) +
    StrToInt(Piece(B, '.', 4));
  Result := NumA - NumB;
end;


function ServerAndDLLVersion(Broker: TRPCBroker): string;
begin
  result := '';

  if not Broker.Connected then
    exit;

  with Broker do
    begin
      ClearResults := True;
      ClearParameters := True;
      CreateContext('YS BROKER1');
      RemoteProcedure := 'YTQ VERSRV';
      Param[0].Mult['"YSB"'] := 'YS BROKER1';

      Param[0].PType := List;
      call;

      if results[0] = '[ERROR]' then
        result := 'ERROR'
      else
        result := Piece(results[1], U, 1) + U +
          Piece(results[2], U, 1); //Server Version Number for Clin Reminders

     // Needed for DLL, in case there is a crash
      CreateContext('OR CPRS GUI CHART');
(*
      ClearResults := True;
      ClearParameters := True;
      RemoteProcedure := 'ORWU VERSRV';
      Param[0].Mult['"X"'] := TX_OPTION;
      Param[0].Mult['"CLVER"'] := ;
      Param[0].PType := List;
      call;
*)
    end;
end;


{$WARN UNSAFE_CODE OFF}
{$WARN UNSAFE_TYPE OFF}
{$WARN UNSAFE_CAST OFF}
function ClientVersion(const AFileName: string): string; //CPRS Code
var
  ASize, AHandle              : DWORD;
  Buf                         : string;
  FileInfoPtr                 : Pointer; //PVSFixedFileInfo;
begin
  Result := '';
  ASize := GetFileVersionInfoSize(PChar(AFileName), AHandle);
  if ASize > 0 then
    begin
      SetLength(Buf, ASize);
      GetFileVersionInfo(PChar(AFileName), AHandle, ASize, Pointer(Buf));
      VerQueryValue(Pointer(Buf), '\', FileInfoPtr, ASize);
      with TVSFixedFileInfo(FileInfoPtr^) do
        Result := IntToStr(HIWORD(dwFileVersionMS)) + '.' +
          IntToStr(LOWORD(dwFileVersionMS)) + '.' +
          IntToStr(HIWORD(dwFileVersionLS)) + '.' +
          IntToStr(LOWORD(dwFileVersionLS));
    end;
end;


{$WARN UNSAFE_CAST ON}
{$WARN UNSAFE_TYPE ON}
{$WARN UNSAFE_CODE ON}

function FileVersionValue(const AFileName, AValueName: string): string;
type
  PValBuf = ^TValBuf;
  TValBuf = array[0..255] of Char;
var
  VerSize, ValSize, AHandle   : DWORD;
  VerBuf                      : Pointer;
  ValBuf                      : PValBuf;
begin
  Result := '';
  VerSize := GetFileVersionInfoSize(PChar(AFileName), AHandle);
  if VerSize > 0 then
    begin
      GetMem(VerBuf, VerSize);
      try
        GetFileVersionInfo(PChar(AFileName), AHandle, VerSize, VerBuf);
        VerQueryValue(VerBuf, PChar(AValueName), Pointer(ValBuf), ValSize);
        SetString(Result, ValBuf^, ValSize);
      finally
        FreeMem(VerBuf);
      end;
    end;
end;


function CorrectVersion(Broker: TRPCBroker): Boolean;
var
  TempStr,
    ServerStr,
    RequiredDLLVersion,
    CurrentDLLVersion,
    ServerNeededStr           : string;
begin
  result := False;

  ServerNeededStr := Piece(FileVersionValue(GetModuleName(HInstance),
    FILE_VER_INTERNALNAME), ' ', 1);
  TempStr := ServerAndDLLVersion(Broker);
  ServerStr := Piece(TempStr, U, 1);

  RequiredDLLVersion := Piece(TempStr, U, 2);
  CurrentDLLVersion := ClientVersion(GetModuleName(HInstance));

// ShowMessage( ServerStr+#13+
//    RequiredDLLVersion+#13+
//    CurrentDLLVersion+#13+
//    ServerNeededStr);

  // When connection to server is broken
  if ServerStr = '' then
    Exit;

  // When broker call is trashed
  if ServerStr = 'ERROR' then
    begin
      MsgDlgFrm(
        'The file, ' + ExtractFileName(GetModuleName(HInstance)) + ', is having trouble connecting ' +
        'to broker to read the version of the server. Please contact your local ' +
        'IRM or CPRS expert for help. ',
        MHA_DLL_CAPTION, mtError, [mbOK], 0);
      Exit;
    end;

   // When broker call is trashed
  if ServerNeededStr = '' then
    begin
      MsgDlgFrm(
        'The file, ' + ExtractFileName(GetModuleName(HInstance)) + ', is having trouble identifying the ' +
        'version of the broker it works with. Please contact your local ' +
        'IRM or CPRS expert for help. ',
        MHA_DLL_CAPTION, mtError, [mbOK], 0);
      Exit;
    end;

  if (CompareVersion(ServerStr, ServerNeededStr) > 0) then
    begin
      MsgDlgFrm(TX_VER1 + CurrentDLLVersion + ' of ' + ExtractFileName(GetModuleName(HInstance)) + #13 +
        ServerNeededStr + TX_VER_REQ + TX_VER3 + ServerStr + '.' +
        TX_VER_OLD2,
        MHA_DLL_CAPTION, mtError, [mbOK], 0);
      Exit;
    end;

  // If Server older than Required, then Server needs updating
  if (CompareVersion(ServerStr, ServerNeededStr) < 0) then
    begin
      MsgDlgFrm(TX_VER1 + CurrentDLLVersion + ' of ' + ExtractFileName(GetModuleName(HInstance)) + #13 +
        ServerNeededStr + TX_VER_REQ + TX_VER3 + ServerStr + '.' +
        TX_VER_NEW,
        MHA_DLL_CAPTION, mtError, [mbOK], 0);
      Exit;
    end;                                             

  // This version of the DLL is outdated
  if (CompareVersion(CurrentDLLVersion, RequiredDLLVersion) < 0) then
    begin
      MsgDlgFrm(TX_VER1 + CurrentDLLVersion + ' of ' + ExtractFileName(GetModuleName(HInstance)) + #13 +
        RequiredDLLVersion + ' is now the proper version in service. ' +
        'Please contact your local ' +
        'IRM or CPRS expert to obtain the updated version.',
        MHA_DLL_CAPTION, mtError, [mbOK], 0);
      Exit;
    end;

  Result := True;
end;

end.

